#include <iostream>
#include <iomanip>
#include <vector>
#include <fstream>
#include "numerov.h"

using namespace std;

void SolvePoisson(int Zq, const vector<double>& Rmesh, const vector<double>& rho, vector<double>& Uhartree)
{// Given the input density rho, calculates the Hartree potential
  static vector<double> RHS(Rmesh.size());
  for (int i=0; i<Rmesh.size(); i++) RHS[i] = -4*M_PI*Rmesh[i]*rho[i];
  Uhartree[0]=0;  Uhartree[1]=(Rmesh[1]-Rmesh[0]);// Boundary condition for U_H=V_H/r
  NumerovInhom(RHS, RHS.size(), Rmesh[1]-Rmesh[0], Uhartree); // Solving the 2nd order differential equation
  // adding homogeneous solution to satisfay boundary conditions: U(0)=0, U(infinity)=Z
  int ilast = Uhartree.size()-1;
  double U_last = Uhartree[ilast];
  double alpha = (Zq - U_last)/Rmesh[ilast];
  for (int i=0; i<Rmesh.size(); i++) Uhartree[i] += alpha*Rmesh[i];
}


int main(int argc, char *argv[], char *env[])
{
  int Z, N;
  ifstream input("rho.dat");
  string str;
  input>>str;
  input>>Z;
  input>>N;
  vector<double> Rmesh(N), rho(N), Uhartree(N);
  // read input density
  for (int i=0; i<N; i++) input>>Rmesh[i]>>rho[i];

  
  SolvePoisson(Z, Rmesh, rho, Uhartree);// Given the input density rho, calculates the Hartree potential

  for (int i=0; i<Rmesh.size(); i++) cout<<Rmesh[i]<<" "<<Uhartree[i]<<" "<<rho[i]<<endl;
  return 0;
}
